//
//  featureProcessing.h
//  featMatch
//
//  Created by Abe on 25/06/15.
//  Copyright (c) 2015 Abe. All rights reserved.
//

#ifndef featMatch_featureProcessing_h
#define featMatch_featureProcessing_h

/// OpenCV declarations
#include <opencv2/core/core.hpp>
#include <opencv2/features2d/features2d.hpp>
#include <opencv2/highgui/highgui.hpp>
#include "opencv2/calib3d/calib3d.hpp"
#include "opencv2/imgproc/imgproc.hpp"

/// Other declarations
#include <vector>
#include <iostream>

#endif


/*
    Class FeatureProcessing is not compatible with OpenCV 3.0!
    Works with 2.4.x

*/

class FeatureProcessing {
public:
    FeatureProcessing();

    /// Load descriptors and keypoints from pathToFile.
    void loadKpDsc(std::string pathToFile, std::vector<cv::KeyPoint> &outputKeypoints, cv::Mat &outputDescriptors);
    /// Save descriptors and keypoints to pathToFile.
    void saveKpDsc(std::string pathToFile, std::vector<cv::KeyPoint> &inputKeypoints, cv::Mat &inputDescriptors);
    /// Detect keypoints from inputImage by method. Store keypoints into outputKeypoints.
    void detect(std::string method, cv::Mat inputImage, std::vector<cv::KeyPoint> &outputKeypoints);
    /// Extract descriptors from inputImage by method and store them into outputDescriptors.
    void extract(std::string method, cv::Mat inputImage, std::vector<cv::KeyPoint> &inputKeypoints, cv::Mat &outputDescriptors);
    /// Match descriptors stored in input1 and input2 and store results into outputMatches. match() takes just "good" matchers and stores them into outputMatches.
    /// Method uses 2 different keypoint matching (depends on brute):
    /// true: brute force matcher - more accurate but slow
    /// false: FLANN matcher - fast, comparatively accurate
    void match(bool brute, cv::Mat &input1, cv::Mat &input2, std::vector<cv::DMatch> &outputMatches);
    /// Draw keypoints in img image on device screen - in window named wnd.
    void drawKeypoints(cv::Mat img, std::vector<cv::KeyPoint> kp, std::string wnd);
    /// Draw found matches among keypoints.
    void drawMatches(cv::Mat img1, std::vector<cv::KeyPoint> key1,cv::Mat img2, std::vector<cv::KeyPoint> key2, std::vector<cv::DMatch> &matches);
    /// Compute transformation matrix between 2 sets of keypoints and store it into matrix.
    void computeTranformationMatrix(std::vector<cv::DMatch> matches, std::vector<cv::KeyPoint> key1, std::vector<cv::KeyPoint> key2, cv::Mat &matrix);
    /// Draw perspectively transformed image img (transformed by matrix).
    void drawPerspectiveImage(cv::Mat img, cv::Mat matrix);
    /// Makes all steps manually. Takes input image, finds keypoints, match them with some stored keypoints and return transformation matrix. This method uses FAST, BRISK and FLANN methods for feature detecting, descripting and matching.
    void run(std::string pathToFile, cv::Mat inputImage, cv::Mat &matrix);
    /// Same as above; scene keypoints detect in image, not from the file.
    void run(cv::Mat inputImage, cv::Mat inputImageScene, cv::Mat &matrix);
    /// Makes all steps manually. Takes input image, finds keypoints, match them with some stored keypoints and return transformation matrix. Methods for feature detecting, descripting and matching must be defined
    void run(std::string pathToFile, cv::Mat inputImage, cv::Mat &matrix, std::string methodDet, std::string methodExt, bool brute);
    /// print image to device screen
    void printImage(std::string windowName, cv::Mat printedImage);


};
